# Copyright 2023 DARWIN EU®
#
# This file is part of ReportGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
utils::globalVariables(c("cdmTableName",
                         "cdmFieldName",
                         "createCdmDeath",
                         "db",
                         ".SD",
                         "concept_name",
                         "domain_id",
                         "subject_id",
                         "cohort_start_date",
                         "cohort",
                         "cohort_end_date",
                         "delete_event",
                         "person_id",
                         "setNames",
                         "tail",
                         "updateTableDates"))
