% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patientChat_naive.R
\name{availableModels}
\alias{availableModels}
\title{\code{availableModels()} If the API key is valid in the system, returns available models to the user from the LLM provider.}
\usage{
availableModels()
}
\value{
A string list with the id of a vailable models.
}
\description{
OpenAI is the only one currently supported.
}
