% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilsPatientDesigner.R
\name{getTestSets}
\alias{getTestSets}
\title{Useful to list available datasets in the testCases folder}
\usage{
getTestSets(path = NULL)
}
\arguments{
\item{path}{Optional directory containing JSON test sets.
If NULL, the package resolves a default path with testthat integration.}
}
\value{
A list()
}
\description{
Useful to list available datasets in the testCases folder
}
