% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patientsDesigner.R
\name{patientDesigner}
\alias{patientDesigner}
\title{\code{patientDesigner()} is a visual interface based on D3 to construct test datasets for the OMOP-CDM}
\usage{
patientDesigner(path = NULL)
}
\arguments{
\item{path}{Optional folder containing JSON test sets.
If NULL, default path resolution keeps testthat integration.}
}
\value{
A Shiny app
}
\description{
\code{patientDesigner()} is a visual interface based on D3 to construct test datasets for the OMOP-CDM
}
