% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hecateSearch.R
\name{hecateClient}
\alias{hecateClient}
\title{Create a Hecate API client}
\usage{
hecateClient(baseUrl = NULL, timeoutMs = NULL, apiKey = NULL)
}
\arguments{
\item{baseUrl}{Base URL of the Hecate API (default from config).}

\item{timeoutMs}{Timeout in milliseconds (default from config).}

\item{apiKey}{Optional API key for authorization (default from config).}
}
\value{
A client object with class \code{hecate_client}.
}
\description{
Create a Hecate API client
}
