% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hecateSearch.R
\name{nullOr}
\alias{nullOr}
\alias{\%||\%}
\title{Null coalescing operator}
\usage{
x \%||\% y
}
\arguments{
\item{x}{First value (any type).}

\item{y}{Fallback value when \code{x} is NULL.}
}
\value{
\code{x} if not NULL, otherwise \code{y}.
}
\description{
Null coalescing operator
}
